<?php

namespace Drupal\microspid\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Link;
use Drupal\Core\Url;
use \Drupal\block\entity\Block;

/**
 * Form builder for the microspid basic settings form.
 */
class BasicSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'microspid_basic_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['microspid.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    global $base_url;
    $config = $this->config('microspid.settings');
    $certman = \Drupal::service('microspid.certs.manager');
    
    $form['basic'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Basic settings'),
      '#collapsible' => FALSE,
    ];
    $desc = $this->t('Click to activate SPID. ');
    $exists = $certman->certExists();
    if (!$exists) {
      $desc .= Link::fromTextAndUrl($this->t('Create certificate'), Url::fromUri('internal:/admin/config/people/microspid/create_cert'))->toString();
    }
    $form['basic']['activate'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Activate authentication via MicroSPID'),
      '#default_value' => $config->get('activate'),
      '#description' => $desc,
    ];
    $form['basic']['spid-button'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Attiva bottone SPID'),
      '#default_value' => FALSE,
      '#description' => $this->t('Selezionando questa casella verrà attivato il bottone SPID se non già presente.'),
    ];
    $form['basic']['authlevel'] = array(
      '#type' => 'radios',
      '#title' => t('Livello di autenticazione SPID'),
      '#default_value' => $config->get('authlevel'),
      '#description' => t('Autenticazione livello 1 o 2'),
      '#options' => array(
        'SpidL1' => 'Livello 1',
        'SpidL2' => 'Livello 2',
      ),
    );
    $form['basic']['index'] = array(
      '#type' => 'textfield',
      '#attributes' => array(
    // Insert space before attribute name :)
        ' type' => 'number',
      ),
      '#title' => t('Service index'),
      '#default_value' => $config->get('index'),
      '#maxlength' => 2,
      '#size' => 2,
      '#description' => t('Specify service index'),
    );
    $defval = @$config->get('entityid');
    if (empty($defval)) $defval = $base_url . '/microspid_metadata';
    $form['basic']['entityid'] = array(
      '#type' => 'textfield',
      '#title' => t('SP Entity ID'),
      '#default_value' => $defval,
      '#description' => t('If service is 0 leave unchanged else use service 0 value'),
    );
    $form['basic']['servicedir'] = array(
      '#type' => 'textfield',
      '#title' => t('Service path'),
      '#default_value' => $config->get('servicedir'),
      '#description' => t('Service path/alias'),
    );
    $form['basic']['privatepath'] = array(
      '#type' => 'textfield',
      '#title' => t('private key path'),
      '#default_value' => $config->get('privatepath'),
      '#description' => t('private key path, only if you change it'),
      '#element_validate' => array(array($this, 'validatePrivatepath')),
    );
    $form['basic']['header_no_cache'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use Header with: Cache-Control: no-cache'),
      '#default_value' => $config->get('header_no_cache'),
      '#description' => $this->t('Use a "Cache-Control: no-cache" header in the HTTP response to avoid the redirection be cached (e.g. when using a reverse-proxy layer).'),
    ];
    $form['basic']['single_logout'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Do Single Logout'),
      '#default_value' => $config->get('single_logout'),
      '#description' => $this->t('Enable Single Logout (only for level 1).'),
    ];
    $form['basic']['test_mode'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Test mode'),
      '#default_value' => $config->get('test_mode'),
      '#description' => $this->t('Use testing idp.'),
    ];

    $form['debugging'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Debugging'),
      '#collapsible' => FALSE,
    ];
    $form['debugging']['debug'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Turn on debugging messages'),
      '#default_value' => $config->get('debug'),
      '#description' => $this->t('Expand the level of Drupal logging to include debugging information.'),
    ];

    $form['user_provisioning'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('User Provisioning'),
      '#collapsible' => FALSE,
    ];
    $form['user_provisioning']['register_users'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Register users (i.e., auto-provisioning)'),
      '#default_value' => $config->get('register_users'),
      '#description' => $this->t('Determines whether or not the module should automatically create/register new Drupal accounts for users that authenticate using MicroSPiD. Unless you\'ve done some custom work to provision Drupal accounts with the necessary authmap entries you will want this checked.<br /><br />NOTE: If unchecked each user must already have been provisioned a Drupal account correctly linked to the SAML authname attribute (e.g. by creating Drupal users with "Enable this user to leverage SAML authentication" checked). Otherwise they will receive a notice and be denied access.'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $config = $this->config('microspid.settings');

    $config->set('activate', $form_state->getValue('activate'));
    $config->set('index', $form_state->getValue('index'));
    $config->set('entityid', $form_state->getValue('entityid'));
    $config->set('servicedir', $form_state->getValue('servicedir'));
    $config->set('privatepath', $form_state->getValue('privatepath'));
    $config->set('authlevel', $form_state->getValue('authlevel'));
    //$config->set('login_link_display_name', $form_state->getValue('login_link_display_name'));
    $config->set('single_logout', $form_state->getValue('single_logout'));
    $config->set('test_mode', $form_state->getValue('test_mode'));
    $config->set('debug', $form_state->getValue('debug'));
    $config->set('register_users', $form_state->getValue('register_users'));
    $config->set('header_no_cache', $form_state->getValue('header_no_cache'));
    $config->save();
    if ($form_state->getValue('spid-button')) {
      $this->saveBlock();
    }
    Cache::invalidateTags(['rendered']);
  }

  public function validatePrivatepath($element, &$form_state, $form) {
    $filepath = $element['#value'];
    if (!empty($filepath) && !file_exists($filepath)) {
      $form_state->setError($element, $this->t("Folder %filepath doesn't exist", array('%filepath' => $filepath)));
    }
  }

  public function saveBlock() {
    $blockEntityManager = \Drupal::service('entity.manager')->getStorage('block');
    $theme = \Drupal::config('system.theme')->get('default');
    $plugin_id = 'microspid_block';
    $vis_config = 
      [
        'request_path' => 
        [
           'id' => 'request_path',
           'pages' =>  '/user/login',
           'negate' => 0,
           'context_mapping' => [],
        ]
      ];

    $my_block = Block::load('spidauthstatus');
    if ($my_block && $my_block->getTheme() == $theme) {
        return;
    }
    if ($my_block) {
        $my_block->delete();
    }
	
    $my_block = $blockEntityManager->create(
      array(
        'id'=> 'spidauthstatus',
        'plugin' => $plugin_id,
	    'settings' => [
	      'id' => 'microspid_block',
		  'label' => t('Otherwise'),
		  'provider' => 'microspid',
		  'label_display' => 'visible',
	    ],
	    'region' => 'content',
        'theme' => $theme,
	    'visibility' => $vis_config,
	    'weight' => 100,
      )
    );
    $my_block->save();
  }

}
